@@ -1,8 +1,8 @@
 ##############################################################################
-#      $URL: http://perlcritic.tigris.org/svn/perlcritic/tags/Perl-Critic-1.092/lib/Perl/Critic/Policy.pm $
-#     $Date: 2008-09-02 09:43:48 -0700 (Tue, 02 Sep 2008) $
+#      $URL: http://perlcritic.tigris.org/svn/perlcritic/trunk/distributions/Perl-Critic/lib/Perl/Critic/Policy.pm $
+#     $Date: 2012-07-02 22:16:39 -0700 (Mon, 02 Jul 2012) $
 #   $Author: thaljef $
-# $Revision: 2721 $
+# $Revision: 4126 $
 ##############################################################################
 
 package Perl::Critic::Policy;
@@ -47,7 +47,7 @@ use Perl::Critic::Violation qw<>;
 
 use Exception::Class;   # this must come after "use P::C::Exception::*"
 
-our $VERSION = '1.092';
+our $VERSION = '1.118';
 
 #-----------------------------------------------------------------------------
 
@@ -55,7 +55,7 @@ Readonly::Scalar my $NO_LIMIT => 'no_limit';
 
 #-----------------------------------------------------------------------------
 
-my $FORMAT = "%p\n"; #Default stringy format
+my $format = "%p\n"; #Default stringy format
 
 #-----------------------------------------------------------------------------
 
@@ -104,7 +104,7 @@ sub new {
     }
 
     if ($parameter_metadata_available) {
-        $self->_validate_config_keys($errors, $config_object);
+        $config_object->handle_extra_parameters( $self, $errors );
     }
 
     if ( $errors->has_exceptions() ) {
@@ -116,26 +116,20 @@ sub new {
 
 #-----------------------------------------------------------------------------
 
+sub is_safe {
+    return $TRUE;
+}
+
+#-----------------------------------------------------------------------------
+
 sub initialize_if_enabled {
     return $TRUE;
 }
 
 #-----------------------------------------------------------------------------
 
-sub _validate_config_keys {
-    my ( $self, $errors, $config ) = @_;
-
-    for my $offered_param ( $config->get_parameter_names() ) {
-        $errors->add_exception(
-            Perl::Critic::Exception::Configuration::Option::Policy::ExtraParameter->new(
-                policy          => $self->get_short_name(),
-                option_name     => $offered_param,
-                source          => undef,
-            )
-        );
-    }
-
-    return;
+sub prepare_to_scan_document {
+    return $TRUE;
 }
 
 #-----------------------------------------------------------------------------
@@ -272,6 +266,24 @@ sub get_short_name {
 
 #-----------------------------------------------------------------------------
 
+sub is_enabled {
+    my ($self) = @_;
+
+    return $self->{_enabled};
+}
+
+#-----------------------------------------------------------------------------
+
+sub __set_enabled {
+    my ($self, $new_value) = @_;
+
+    $self->{_enabled} = $new_value;
+
+    return;
+}
+
+#-----------------------------------------------------------------------------
+
 sub applies_to {
     return qw(PPI::Element);
 }
@@ -337,8 +349,9 @@ sub set_themes {
 
 sub get_themes {
     my ($self) = @_;
-    return sort @{ $self->{_themes} } if defined $self->{_themes};
-    return sort $self->default_themes();
+    my @themes = defined $self->{_themes} ? @{ $self->{_themes} } : $self->default_themes();
+    my @sorted_themes = sort @themes;
+    return @sorted_themes;
 }
 
 #-----------------------------------------------------------------------------
@@ -400,7 +413,7 @@ sub violates {
 
 #-----------------------------------------------------------------------------
 
-sub violation {  ##no critic(ArgUnpacking)
+sub violation {  ## no critic (ArgUnpacking)
     my ( $self, $desc, $expl, $elem ) = @_;
     # HACK!! Use goto instead of an explicit call because P::C::V::new() uses caller()
     my $sev = $self->get_severity();
@@ -422,7 +435,6 @@ sub new_parameter_value_exception {
     );
 }
 
-
 #-----------------------------------------------------------------------------
 
 ## no critic (Subroutines::RequireFinalReturn)
@@ -441,8 +453,8 @@ sub throw_parameter_value_exception {
 
 # Static methods.
 
-sub set_format { return $FORMAT = $_[0] }  ##no critic(ArgUnpacking)
-sub get_format { return $FORMAT         }
+sub set_format { return $format = $_[0] }  ## no critic(ArgUnpacking)
+sub get_format { return $format         }
 
 #-----------------------------------------------------------------------------
 
@@ -463,26 +475,26 @@ sub to_string {
          'V' => sub { dor( $self->default_maximum_violations_per_document(), $NO_LIMIT ) },
          'v' => sub { dor( $self->get_maximum_violations_per_document(), $NO_LIMIT ) },
     );
-    return stringf($FORMAT, %fspec);
+    return stringf(get_format(), %fspec);
 }
 
 sub _format_parameters {
-    my ($self, $format) = @_;
+    my ($self, $parameter_format) = @_;
 
     return $EMPTY if not $self->parameter_metadata_available();
 
     my $separator;
-    if ($format) {
+    if ($parameter_format) {
         $separator = $EMPTY;
     } else {
         $separator = $SPACE;
-        $format = '%n';
+        $parameter_format = '%n';
     }
 
     return
         join
             $separator,
-            map { $_->to_formatted_string($format) } @{ $self->get_parameters() };
+            map { $_->to_formatted_string($parameter_format) } @{ $self->get_parameters() };
 }
 
 sub _format_lack_of_parameter_metadata {
@@ -495,16 +507,6 @@ sub _format_lack_of_parameter_metadata {
         'Cannot programmatically discover what parameters this policy takes.';
 }
 
-sub _get_source_file {
-    my ($self) = @_;
-
-    my $relative_path =
-        File::Spec->catfile( split m/::/xms, ref $self ) . '.pm';
-
-    return $INC{$relative_path};
-}
-
-
 #-----------------------------------------------------------------------------
 # Apparently, some perls do not implicitly stringify overloading
 # objects before doing a comparison.  This causes a couple of our
@@ -544,28 +546,28 @@ L<Perl::Critic::DEVELOPER|Perl::Critic::DEVELOPER> document included
 in this distribution.
 
 
+=head1 INTERFACE SUPPORT
+
+This is considered to be a public class.  Any changes to its interface
+will go through a deprecation cycle.
+
+
 =head1 METHODS
 
 =over
 
-=item C<< new(key1 => value1, key2 => value2 ... ) >>
+=item C<< new( ... ) >>
 
-Returns a reference to a new subclass of Perl::Critic::Policy. If your
-Policy requires any special arguments, they will be passed in here as
-key-value pairs.  Users of L<perlcritic|perlcritic> can specify these
-in their config file.  Unless you override the C<new> method, the
-default method simply returns a reference to an empty hash that has
-been blessed into your subclass.  However, you really should not
-override this; override C<initialize_if_enabled()> instead.
-
-This constructor is always called regardless of whether the user has
-enabled this Policy or not.
+Don't call this.  As a Policy author, do not implement this.  Use the
+C<initialize_if_enabled()> method for your Policy setup.  See the
+L<developer|Perl::Critic::DEVELOPER> documentation for more.
 
 
-=item C<< initialize_if_enabled( { key1 => value1, key2 => value2 ... } ) >>
+=item C<< initialize_if_enabled( $config ) >>
 
-This receives the same parameters as C<new()>, but as a reference to a
-hash, and is only invoked if this Policy is enabled by the user.
+This receives an instance of
+L<Perl::Critic::PolicyConfig|Perl::Critic::PolicyConfig> as a
+parameter, and is only invoked if this Policy is enabled by the user.
 Thus, this is the preferred place for subclasses to do any
 initialization.
 
@@ -577,6 +579,15 @@ available should test for the availability of these dependencies and
 return C<$FALSE> if they are not.
 
 
+=item C<< prepare_to_scan_document( $document ) >>
+
+The parameter is about to be scanned by this Policy.  Whatever this
+Policy wants to do in terms of preparation should happen here.
+Returns a boolean value indicating whether the document should be
+scanned at all; if this is a false value, this Policy won't be applied
+to the document.  By default, does nothing but return C<$TRUE>.
+
+
 =item C< violates( $element, $document ) >
 
 Given a L<PPI::Element|PPI::Element> and a
@@ -630,6 +641,13 @@ Return the name of this policy without the "Perl::Critic::Policy::"
 prefix.
 
 
+=item C< is_enabled() >
+
+Answer whether this policy is really active or not.  Returns a true
+value if it is, a false, yet defined, value if it isn't, and an
+undefined value if it hasn't yet been decided whether it will be.
+
+
 =item C< applies_to() >
 
 Returns a list of the names of PPI classes that this Policy cares
@@ -649,8 +667,8 @@ override this.
 
 Returns the maximum number of violations this policy will report for a
 single document.  If this is not defined, then there is no limit.  If
-L<set_maximum_violations_per_document()> has not been invoked, then
-L<default_maximum_violations_per_document()> is returned.
+L</set_maximum_violations_per_document()> has not been invoked, then
+L</default_maximum_violations_per_document()> is returned.
 
 
 =item C< set_maximum_violations_per_document() >
@@ -746,14 +764,7 @@ circumstance and the one where this policy does not take any
 parameters, it is necessary to call C<parameter_metadata_available()>.
 
 
-=item C< get_parameter( $parameter_name ) >
-
-Returns the
-L<Perl::Critic::PolicyParameter|Perl::Critic::PolicyParameter> with
-the specified name.
-
-
-=item C<set_format( $FORMAT )>
+=item C<set_format( $format )>
 
 Class method.  Sets the format for all Policy objects when they are
 evaluated in string context.  The default is C<"%p\n">.  See
@@ -769,8 +780,24 @@ they are evaluated in string context.
 =item C<to_string()>
 
 Returns a string representation of the policy.  The content of the
-string depends on the current value of the C<$FORMAT> package
-variable.  See L<"OVERLOADS"> for the details.
+string depends on the current value returned by C<get_format()>.
+See L<"OVERLOADS"> for the details.
+
+
+=item C<is_safe()>
+
+Answer whether this Policy can be used to analyze untrusted code, i.e. the
+Policy doesn't have any potential side effects.
+
+This method returns a true value by default.
+
+An "unsafe" policy might attempt to compile the code, which, if you have
+C<BEGIN> or C<CHECK> blocks that affect files or connect to databases, is not
+a safe thing to do.  If you are writing a such a Policy, then you should
+override this method to return false.
+
+By default L<Perl::Critic|Perl::Critic> will not run unsafe policies.
+
 
 
 =back
@@ -789,8 +816,7 @@ POD for any Policy modules that you author.  Thanks.
 =head1 OVERLOADS
 
 Perl::Critic::Violation overloads the C<""> operator to produce neat
-little messages when evaluated in string context.  The format depends
-on the current value of the C<$FORMAT> package variable.
+little messages when evaluated in string context.
 
 Formats are a combination of literal and escape characters similar to
 the way C<sprintf> works.  If you want to know the specific formatting
@@ -868,12 +894,12 @@ The current maximum number of violations per document of the policy.
 
 =head1 AUTHOR
 
-Jeffrey Ryan Thalhammer <thaljef@cpan.org>
+Jeffrey Ryan Thalhammer <jeff@imaginative-software.com>
 
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005-2008 Jeffrey Ryan Thalhammer.  All rights reserved.
+Copyright (c) 2005-2011 Imaginative Software Systems.  All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.  The full text of this license
